@@ -1,9 +1,9 @@
 package App::GoogleSearchUtils;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-10-01'; # DATE
+our $DATE = '2020-10-14'; # DATE
 our $DIST = 'App-GoogleSearchUtils'; # DIST
-our $VERSION = '0.004'; # VERSION
+our $VERSION = '0.005'; # VERSION
 
 use 5.010001;
 use strict;
@@ -43,6 +43,29 @@ $SPEC{google_search} = {
             schema => 'posint*',
             default => 100,
         },
+        action => {
+            summary => 'What to do with the URLs',
+            schema => ['str*', in=>[qw/open_url print_url print_html_link print_org_link/]],
+            default => 'open_url',
+            cmdline_aliases => {
+                open_url        => {is_flag=>1, summary=>'Alias for --action=open_url'       , code=>sub {$_[0]{action}='open_url'       }},
+                print_url       => {is_flag=>1, summary=>'Alias for --action=print_url'      , code=>sub {$_[0]{action}='print_url'      }},
+                print_html_link => {is_flag=>1, summary=>'Alias for --action=print_html_link', code=>sub {$_[0]{action}='print_html_link'}},
+                print_org_link  => {is_flag=>1, summary=>'Alias for --action=print_org_link' , code=>sub {$_[0]{action}='print_org_link' }},
+            },
+        },
+        type => {
+            summary => 'Search type',
+            schema => ['str*', in=>[qw/web image video news map/]],
+            default => 'web',
+            cmdline_aliases => {
+                web   => {is_flag=>1, summary=>'Alias for --type=web'  , code=>sub {$_[0]{type}='web'  }},
+                image => {is_flag=>1, summary=>'Alias for --type=image', code=>sub {$_[0]{type}='image'}},
+                video => {is_flag=>1, summary=>'Alias for --type=video', code=>sub {$_[0]{type}='video'}},
+                news  => {is_flag=>1, summary=>'Alias for --type=news' , code=>sub {$_[0]{type}='news' }},
+                map   => {is_flag=>1, summary=>'Alias for --type=map'  , code=>sub {$_[0]{type}='map'  }},
+            },
+        },
     },
     examples => [
         {
@@ -73,6 +96,13 @@ $SPEC{google_search} = {
             test => 0,
             'x.doc.show_result' => 0,
         },
+        {
+            summary => 'Show image search URLs instead of opening them in browser',
+            src => '[[prog]] --image --print "query one" query2',
+            src_plang => 'bash',
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
     ],
 };
 sub google_search {
@@ -81,8 +111,11 @@ sub google_search {
 
     my %args = @_;
     # XXX schema
-    my $num = $args{num} + 0;
+    my $num = defined($args{num}) ? $args{num} + 0 : 100;
+    my $action = $args{action} // 'web';
+    my $type = $args{type} // 'web';
 
+    my @rows;
     my $envres = envresmulti();
     my $i = -1;
     for my $query0 (@{ $args{queries} }) {
@@ -97,13 +130,43 @@ sub google_search {
             $query0,
             defined($args{append}) ? $args{append} : "",
         );
-        my $url = "https://www.google.com/search?num=$num&q=".
-            URI::Escape::uri_escape($query);
-        my $res = Browser::Open::open_browser($url);
-        $envres->add_result(
-            ($res ? (500, "Failed") : (200, "OK")), {item_id=>$i});
+        my $query_esc = URI::Escape::uri_escape($query);
+        my $url;
+        if ($type eq 'web') {
+            $url = "https://www.google.com/search?num=$num&q=$query_esc";
+        } elsif ($type eq 'image') {
+            $url = "https://www.google.com/search?num=$num&q=$query_esc&tbm=isch";
+        } elsif ($type eq 'video') {
+            $url = "https://www.google.com/search?num=$num&q=$query_esc&tbm=isch";
+        } elsif ($type eq 'news') {
+            $url = "https://www.google.com/search?num=$num&q=$query_esc&tbm=nws";
+        } elsif ($type eq 'map') {
+            $url = "https://www.google.com/maps/search/$query_esc/";
+        } else {
+            return [400, "Unknown type '$type'"];
+        }
+
+        if ($action eq 'open_url') {
+            my $res = Browser::Open::open_browser($url);
+            $envres->add_result(
+                ($res ? (500, "Failed") : (200, "OK")), {item_id=>$i});
+        } elsif ($action eq 'print_url') {
+            push @rows, $url;
+        } elsif ($action eq 'print_html_link') {
+            require HTML::Entities;
+            my $query_htmlesc = HTML::Entities::encode_entities($query);
+            push @rows, qq(<a href="$url">$query_htmlesc<</a>);
+        } elsif ($action eq 'print_org_link') {
+            push @rows, qq([[$url][$query]]);
+        } else {
+            return [400, "Unknown action '$action'"];
+        }
+    }
+    if ($action eq 'open_url') {
+        return $envres->as_struct;
+    } else {
+        return [200, "OK", \@rows];
     }
-    $envres->as_struct;
 }
 
 1;
@@ -121,7 +184,7 @@ App::GoogleSearchUtils - CLI utilites related to google searching
 
 =head1 VERSION
 
-This document describes version 0.004 of App::GoogleSearchUtils (from Perl distribution App-GoogleSearchUtils), released on 2020-10-01.
+This document describes version 0.005 of App::GoogleSearchUtils (from Perl distribution App-GoogleSearchUtils), released on 2020-10-14.
 
 =head1 SYNOPSIS
 
@@ -150,6 +213,10 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
+=item * B<action> => I<str> (default: "open_url")
+
+What to do with the URLs.
+
 =item * B<append> => I<str>
 
 String to add at the end of each query.
@@ -168,6 +235,10 @@ String to add at the beginning of each query.
 
 =item * B<queries>* => I<array[str]>
 
+=item * B<type> => I<str> (default: "web")
+
+Search type.
+
 
 =back
 
